;;;			     tui-ipc.el
;;;
;;;	      Copyright (C) 1997       OKUJI Yoshinori
;;;
;;;		   This emacs lisp library conforms
;;;		GNU GENERAL PUBLIC LICENSE Version 2.
;;;
;;; Author:  OKUJI Yoshinori <okuji@kmc.kyoto-u.ac.jp>
;;; Created: May    15, 1997
;;; Revised: Jun     6, 1997
;;;

;;;
;;; Loading.
;;;
(provide 'tui-ipc)
(require 'tuitate)

;;;
;;; System variables.
;;;
(defvar tuitate-ipc-receive-buffer "")
(defvar tuitate-ipc-receive-flag   nil)

;;;
;;; Constant variables.
;;;
(defconst tuitate-ipc-process-name   "tuitate client")
(defconst tuitate-ipc-process-buffer " *tuitate process*")

;;;
;;; IPC subroutines. Correspoding to TUP(TUitate Protocol) version 4.
;;;

;;;
;;; Send a message to the tuitate server(tuitated).
;;;
(defun tuitate-ipc-send (string)
  (save-excursion
    (set-buffer tuitate-ipc-process-buffer)
    (goto-char (point-max))
    (insert "> " string "\n"))
  (process-send-string tuitate-ipc-process-name string))

(defun tuitate-ipc-send-quit ()
  (tuitate-ipc-send "QUIT\n"))

(defun tuitate-ipc-send-game (name mail)
  (tuitate-ipc-send (concat "GAME " name " " mail "\n")))

(defun tuitate-ipc-send-challengerlist ()
  (tuitate-ipc-send "CHALLENGERLIST\n"))

(defun tuitate-ipc-send-challenge (num)
  (tuitate-ipc-send (concat "CHALLENGE " (number-to-string num) "\n")))

(defun tuitate-ipc-send-accept ()
  (tuitate-ipc-send "ACCEPT\n"))

(defun tuitate-ipc-send-refuse ()
  (tuitate-ipc-send "REFUSE\n"))

(defun tuitate-ipc-send-move (sx sy dx dy &optional promotion)
  (tuitate-ipc-send (concat "MOVE "
			    (number-to-string sx) " "
			    (number-to-string sy) " "
			    (number-to-string dx) " "
			    (number-to-string dy) " "
			    (if promotion "PROMOTE\n" "\n"))))

(defun tuitate-ipc-send-drop (dx dy piece)
  (tuitate-ipc-send (concat "DROP "
			    (number-to-string dx) " "
			    (number-to-string dy) " "
			    piece "\n")))

(defun tuitate-ipc-send-giveup ()
  (tuitate-ipc-send "GIVEUP\n"))

(defun tuitate-ipc-send-view ()
  (tuitate-ipc-send "VIEW\n"))

(defun tuitate-ipc-send-gamelist ()
  (tuitate-ipc-send "GAMELIST\n"))

(defun tuitate-ipc-send-viewgame (num)
  (tuitate-ipc-send (concat "VIEWGAME " (number-to-string num) "\n")))

(defun tuitate-ipc-send-notation ()
  (tuitate-ipc-send "NOTATION\n"))

(defun tuitate-ipc-send-notationlist ()
  (tuitate-ipc-send "NOTATIONLIST\n"))

(defun tuitate-ipc-send-viewnotation (num)
  (tuitate-ipc-send (concat "VIEWNOTATION " (number-to-string num) "\n")))

(defun tuitate-ipc-send-cheat (x y)
  (tuitate-ipc-send (concat "CHEAT "
			    (number-to-string x)
			    " "
			    (number-to-string y)
			    "\n")))

;;;
;;; Receive some messages from the tuitate server(tuitated).
;;;

;; Search a newline, and return its index.
(defun tuitate-ipc-search-newline (string)
  (catch 'tuitate-ipc-search-newline
    (let ((i 0) (len (length string)))
      (while (< i len)
	(if (eq (aref string i) ?\n)
	    (throw 'tuitate-ipc-search-newline i))
	(setq i (1+ i))))
    nil))

;; Split a message to words.
(defun tuitate-ipc-split (string)
  (let ((start 0) (end 0) (len (length string)) (word t) ret)
    (while (< end len)
      (if word
	  (if (or (eq (aref string end) ? )
		  (eq (aref string end) ?\t)
		  (eq end (1- len)))
	      (progn
		(setq ret
		      (append ret (cons (substring string start end) nil) nil))
		(setq word nil)))
	(if (not (or (eq (aref string end) ? )
		     (eq (aref string end) ?\t)))
	    (progn
	      (setq start end)
	      (setq word t))))
      (setq end (1+ end)))
    ret))

;; Queuing receiving string, and return a message.
(defun tuitate-ipc-receive (string)
  (save-excursion
    (set-buffer tuitate-ipc-process-buffer)
    (goto-char (point-max))
    (insert "< " string "\n"))
  (setq tuitate-ipc-receive-buffer (concat tuitate-ipc-receive-buffer string))
  (if tuitate-ipc-receive-flag
      (let ((end (tuitate-ipc-search-newline tuitate-ipc-receive-buffer)))
	(if end
	    (prog1
		(tuitate-ipc-split
		 (substring tuitate-ipc-receive-buffer 0 (1+ end)))
	      (setq tuitate-ipc-receive-buffer
		    (substring tuitate-ipc-receive-buffer (1+ end))))
	  nil))))

;;;
;;; Connection, filtering, and shuting down.
;;;

;; Normal process filter.
(defun tuitate-ipc-default-filter (proc string)
  (tuitate-ipc-receive string))

;; Set the process filter.
(defun tuitate-ipc-set-filter (&optional filter)
  (if filter
      (set-process-filter (get-process tuitate-ipc-process-name) filter)
    (set-process-filter (get-process tuitate-ipc-process-name)
			'tuitate-ipc-default-filter)))

;; Shut down.
(defun tuitate-ipc-shutdown ()
  (if (process-status tuitate-ipc-process-name)
      (delete-process tuitate-ipc-process-name)))

;; Connect with the tuitate server(tuitated).
(defun tuitate-ipc-connect ()
  (if (eq (process-status tuitate-ipc-process-name) 'open)
      (progn
	(tuitate-ipc-send-quit)
	(tuitate-ipc-shutdown)))
  (setq tuitate-ipc-receive-flag nil
	tuitate-ipc-receive-buffer "")
  (open-network-stream tuitate-ipc-process-name nil
		       tuitate-server tuitate-port)
  (process-kill-without-query (get-process tuitate-ipc-process-name))
  (tuitate-ipc-set-filter)
  (get-buffer-create tuitate-ipc-process-buffer)
  (save-excursion
    (set-buffer tuitate-ipc-process-buffer)
    (setq buffer-read-only nil)))

;;;
;;; End of tui-ipc.el
;;;
